@@ -1,10 +1,7 @@
-// Don't know how to correctly fix this so lower priority for now.
-#![warn(clippy::vtable_address_comparisons)]
-
-use crate::core::algebra::Vector2;
 use crate::{
     brush::Brush,
     core::{
+        algebra::Vector2,
         color::Color,
         math::{self, Rect, TriangleDefinition},
     },
@@ -12,7 +9,7 @@ use crate::{
     ttf::SharedFont,
     Thickness,
 };
-use std::{any::Any, ops::Deref, ops::Range, sync::Arc};
+use std::{any::Any, ops::Range, sync::Arc};
 
 #[derive(Clone)]
 #[repr(C)]
@@ -45,7 +42,11 @@ impl<T: Any + Sync + Send> From<Arc<T>> for SharedTexture {
 
 impl PartialEq for SharedTexture {
     fn eq(&self, other: &Self) -> bool {
-        std::ptr::eq(self.0.deref(), other.0.deref())
+        // Cast fat pointers to thin first.
+        let ptr_a = &*self.0 as *const _ as *const ();
+        let ptr_b = &*other.0 as *const _ as *const ();
+        // Compare thin pointers.
+        std::ptr::eq(ptr_a, ptr_b)
     }
 }
 
@@ -471,12 +472,7 @@ impl DrawingContext {
         position: Vector2<f32>,
         formatted_text: &FormattedText,
     ) {
-        let font = if let Some(font) = formatted_text.get_font() {
-            font
-        } else {
-            println!("Trying to draw text without font!");
-            return;
-        };
+        let font = formatted_text.get_font();
 
         for element in formatted_text.get_glyphs() {
             let bounds = element.get_bounds();
