@@ -1,7 +1,7 @@
 use crate::{
     border::BorderBuilder,
     brush::{Brush, GradientPoint},
-    button::ButtonBuilder,
+    button::{ButtonBuilder, ButtonMessage},
     canvas::CanvasBuilder,
     core::{
         algebra::Vector2,
@@ -10,23 +10,37 @@ use crate::{
         pool::Handle,
     },
     decorator::DecoratorBuilder,
+    define_constructor,
     grid::{Column, GridBuilder, Row},
-    message::{
-        ButtonMessage, MessageData, MessageDirection, ScrollBarMessage, TextMessage, UiMessage,
-        UiMessageData, WidgetMessage,
-    },
-    text::TextBuilder,
+    message::{MessageDirection, UiMessage},
+    text::{TextBuilder, TextMessage},
     utils::{make_arrow, ArrowDirection},
-    widget::{Widget, WidgetBuilder},
-    BuildContext, Control, HorizontalAlignment, NodeHandleMapping, Orientation, Thickness, UINode,
+    widget::{Widget, WidgetBuilder, WidgetMessage},
+    BuildContext, Control, HorizontalAlignment, NodeHandleMapping, Orientation, Thickness, UiNode,
     UserInterface, VerticalAlignment, BRUSH_LIGHT, BRUSH_LIGHTER, BRUSH_LIGHTEST, COLOR_DARKEST,
     COLOR_LIGHTEST,
 };
-use std::ops::{Deref, DerefMut};
+use std::{
+    any::{Any, TypeId},
+    ops::{Deref, DerefMut},
+};
+
+#[derive(Debug, Clone, PartialEq)]
+pub enum ScrollBarMessage {
+    Value(f32),
+    MinValue(f32),
+    MaxValue(f32),
+}
+
+impl ScrollBarMessage {
+    define_constructor!(ScrollBarMessage:Value => fn value(f32), layout: false);
+    define_constructor!(ScrollBarMessage:MaxValue => fn max_value(f32), layout: false);
+    define_constructor!(ScrollBarMessage:MinValue => fn min_value(f32), layout: false);
+}
 
 #[derive(Clone)]
-pub struct ScrollBar<M: MessageData, C: Control<M, C>> {
-    pub widget: Widget<M, C>,
+pub struct ScrollBar {
+    pub widget: Widget,
     pub min: f32,
     pub max: f32,
     pub value: f32,
@@ -34,18 +48,26 @@ pub struct ScrollBar<M: MessageData, C: Control<M, C>> {
     pub orientation: Orientation,
     pub is_dragging: bool,
     pub offset: Vector2<f32>,
-    pub increase: Handle<UINode<M, C>>,
-    pub decrease: Handle<UINode<M, C>>,
-    pub indicator: Handle<UINode<M, C>>,
-    pub field: Handle<UINode<M, C>>,
-    pub value_text: Handle<UINode<M, C>>,
+    pub increase: Handle<UiNode>,
+    pub decrease: Handle<UiNode>,
+    pub indicator: Handle<UiNode>,
+    pub field: Handle<UiNode>,
+    pub value_text: Handle<UiNode>,
     pub value_precision: usize,
 }
 
-crate::define_widget_deref!(ScrollBar<M, C>);
+crate::define_widget_deref!(ScrollBar);
 
-impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollBar<M, C> {
-    fn resolve(&mut self, node_map: &NodeHandleMapping<M, C>) {
+impl Control for ScrollBar {
+    fn query_component(&self, type_id: TypeId) -> Option<&dyn Any> {
+        if type_id == TypeId::of::<Self>() {
+            Some(self)
+        } else {
+            None
+        }
+    }
+
+    fn resolve(&mut self, node_map: &NodeHandleMapping) {
         node_map.resolve(&mut self.increase);
         node_map.resolve(&mut self.decrease);
         node_map.resolve(&mut self.indicator);
@@ -55,7 +77,7 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollBar<M, C> {
         node_map.resolve(&mut self.field);
     }
 
-    fn arrange_override(&self, ui: &UserInterface<M, C>, final_size: Vector2<f32>) -> Vector2<f32> {
+    fn arrange_override(&self, ui: &UserInterface, final_size: Vector2<f32>) -> Vector2<f32> {
         let size = self.widget.arrange_override(ui, final_size);
 
         // Adjust indicator position according to current value
@@ -124,32 +146,26 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollBar<M, C> {
         size
     }
 
-    fn handle_routed_message(
-        &mut self,
-        ui: &mut UserInterface<M, C>,
-        message: &mut UiMessage<M, C>,
-    ) {
+    fn handle_routed_message(&mut self, ui: &mut UserInterface, message: &mut UiMessage) {
         self.widget.handle_routed_message(ui, message);
 
-        match &message.data() {
-            UiMessageData::Button(ButtonMessage::Click) => {
-                if message.destination() == self.increase {
-                    ui.send_message(ScrollBarMessage::value(
-                        self.handle(),
-                        MessageDirection::ToWidget,
-                        self.value + self.step,
-                    ));
-                } else if message.destination() == self.decrease {
-                    ui.send_message(ScrollBarMessage::value(
-                        self.handle(),
-                        MessageDirection::ToWidget,
-                        self.value - self.step,
-                    ));
-                }
+        if let Some(ButtonMessage::Click) = message.data::<ButtonMessage>() {
+            if message.destination() == self.increase {
+                ui.send_message(ScrollBarMessage::value(
+                    self.handle(),
+                    MessageDirection::ToWidget,
+                    self.value + self.step,
+                ));
+            } else if message.destination() == self.decrease {
+                ui.send_message(ScrollBarMessage::value(
+                    self.handle(),
+                    MessageDirection::ToWidget,
+                    self.value - self.step,
+                ));
             }
-            UiMessageData::ScrollBar(msg)
-                if message.destination() == self.handle()
-                    && message.direction() == MessageDirection::ToWidget =>
+        } else if let Some(msg) = message.data::<ScrollBarMessage>() {
+            if message.destination() == self.handle()
+                && message.direction() == MessageDirection::ToWidget
             {
                 match *msg {
                     ScrollBarMessage::Value(value) => {
@@ -228,99 +244,77 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollBar<M, C> {
                     }
                 }
             }
-            UiMessageData::Widget(msg) => {
-                if message.destination() == self.indicator {
-                    match msg {
-                        WidgetMessage::MouseDown { pos, .. } => {
-                            if self.indicator.is_some() {
-                                let indicator_pos = ui.nodes.borrow(self.indicator).screen_position;
-                                self.is_dragging = true;
-                                self.offset = indicator_pos - *pos;
-                                ui.capture_mouse(self.indicator);
+        } else if let Some(msg) = message.data::<WidgetMessage>() {
+            if message.destination() == self.indicator {
+                match msg {
+                    WidgetMessage::MouseDown { pos, .. } => {
+                        if self.indicator.is_some() {
+                            let indicator_pos = ui.nodes.borrow(self.indicator).screen_position;
+                            self.is_dragging = true;
+                            self.offset = indicator_pos - *pos;
+                            ui.capture_mouse(self.indicator);
+                            message.set_handled(true);
+                        }
+                    }
+                    WidgetMessage::MouseUp { .. } => {
+                        self.is_dragging = false;
+                        ui.release_mouse_capture();
+                        message.set_handled(true);
+                    }
+                    WidgetMessage::MouseMove { pos: mouse_pos, .. } => {
+                        if self.indicator.is_some() {
+                            let canvas =
+                                ui.borrow_by_name_up(self.indicator, ScrollBar::PART_CANVAS);
+                            let indicator_size = ui.nodes.borrow(self.indicator).actual_size();
+                            if self.is_dragging {
+                                let percent = match self.orientation {
+                                    Orientation::Horizontal => {
+                                        let span = canvas.actual_size().x - indicator_size.x;
+                                        let offset =
+                                            mouse_pos.x - canvas.screen_position.x + self.offset.x;
+                                        if span > 0.0 {
+                                            math::clampf(offset / span, 0.0, 1.0)
+                                        } else {
+                                            0.0
+                                        }
+                                    }
+                                    Orientation::Vertical => {
+                                        let span = canvas.actual_size().y - indicator_size.y;
+                                        let offset =
+                                            mouse_pos.y - canvas.screen_position.y + self.offset.y;
+                                        if span > 0.0 {
+                                            math::clampf(offset / span, 0.0, 1.0)
+                                        } else {
+                                            0.0
+                                        }
+                                    }
+                                };
+                                ui.send_message(ScrollBarMessage::value(
+                                    self.handle(),
+                                    MessageDirection::ToWidget,
+                                    self.min + percent * (self.max - self.min),
+                                ));
                                 message.set_handled(true);
                             }
                         }
-                        WidgetMessage::MouseUp { .. } => {
-                            self.is_dragging = false;
-                            ui.release_mouse_capture();
-                            message.set_handled(true);
-                        }
-                        WidgetMessage::MouseMove { pos: mouse_pos, .. } => {
-                            if self.indicator.is_some() {
-                                let canvas = ui.borrow_by_name_up(
-                                    self.indicator,
-                                    ScrollBar::<M, C>::PART_CANVAS,
-                                );
-                                let indicator_size = ui.nodes.borrow(self.indicator).actual_size();
-                                if self.is_dragging {
-                                    let percent = match self.orientation {
-                                        Orientation::Horizontal => {
-                                            let span = canvas.actual_size().x - indicator_size.x;
-                                            let offset = mouse_pos.x - canvas.screen_position.x
-                                                + self.offset.x;
-                                            if span > 0.0 {
-                                                math::clampf(offset / span, 0.0, 1.0)
-                                            } else {
-                                                0.0
-                                            }
-                                        }
-                                        Orientation::Vertical => {
-                                            let span = canvas.actual_size().y - indicator_size.y;
-                                            let offset = mouse_pos.y - canvas.screen_position.y
-                                                + self.offset.y;
-                                            if span > 0.0 {
-                                                math::clampf(offset / span, 0.0, 1.0)
-                                            } else {
-                                                0.0
-                                            }
-                                        }
-                                    };
-                                    ui.send_message(ScrollBarMessage::value(
-                                        self.handle(),
-                                        MessageDirection::ToWidget,
-                                        self.min + percent * (self.max - self.min),
-                                    ));
-                                    message.set_handled(true);
-                                }
-                            }
-                        }
-                        _ => (),
                     }
+                    _ => (),
                 }
             }
-            _ => {}
-        }
-    }
-
-    fn remove_ref(&mut self, handle: Handle<UINode<M, C>>) {
-        if self.indicator == handle {
-            self.indicator = Handle::NONE;
-        }
-        if self.decrease == handle {
-            self.decrease = Handle::NONE;
-        }
-        if self.increase == handle {
-            self.increase = Handle::NONE;
-        }
-        if self.value_text == handle {
-            self.value_text = Handle::NONE;
-        }
-        if self.field == handle {
-            self.field = Handle::NONE;
         }
     }
 }
 
-impl<M: MessageData, C: Control<M, C>> ScrollBar<M, C> {
+impl ScrollBar {
     pub const PART_CANVAS: &'static str = "PART_Canvas";
 
     pub fn new(
-        widget: Widget<M, C>,
-        increase: Handle<UINode<M, C>>,
-        decrease: Handle<UINode<M, C>>,
-        indicator: Handle<UINode<M, C>>,
-        field: Handle<UINode<M, C>>,
-        value_text: Handle<UINode<M, C>>,
+        widget: Widget,
+        increase: Handle<UiNode>,
+        decrease: Handle<UiNode>,
+        indicator: Handle<UiNode>,
+        field: Handle<UiNode>,
+        value_text: Handle<UiNode>,
     ) -> Self {
         Self {
             widget,
@@ -362,23 +356,23 @@ impl<M: MessageData, C: Control<M, C>> ScrollBar<M, C> {
     }
 }
 
-pub struct ScrollBarBuilder<M: MessageData, C: Control<M, C>> {
-    widget_builder: WidgetBuilder<M, C>,
+pub struct ScrollBarBuilder {
+    widget_builder: WidgetBuilder,
     min: Option<f32>,
     max: Option<f32>,
     value: Option<f32>,
     step: Option<f32>,
     orientation: Option<Orientation>,
-    increase: Option<Handle<UINode<M, C>>>,
-    decrease: Option<Handle<UINode<M, C>>>,
-    indicator: Option<Handle<UINode<M, C>>>,
-    body: Option<Handle<UINode<M, C>>>,
+    increase: Option<Handle<UiNode>>,
+    decrease: Option<Handle<UiNode>>,
+    indicator: Option<Handle<UiNode>>,
+    body: Option<Handle<UiNode>>,
     show_value: bool,
     value_precision: usize,
 }
 
-impl<M: MessageData, C: Control<M, C>> ScrollBarBuilder<M, C> {
-    pub fn new(widget_builder: WidgetBuilder<M, C>) -> Self {
+impl ScrollBarBuilder {
+    pub fn new(widget_builder: WidgetBuilder) -> Self {
         Self {
             widget_builder,
             min: None,
@@ -420,22 +414,22 @@ impl<M: MessageData, C: Control<M, C>> ScrollBarBuilder<M, C> {
         self
     }
 
-    pub fn with_increase(mut self, increase: Handle<UINode<M, C>>) -> Self {
+    pub fn with_increase(mut self, increase: Handle<UiNode>) -> Self {
         self.increase = Some(increase);
         self
     }
 
-    pub fn with_decrease(mut self, decrease: Handle<UINode<M, C>>) -> Self {
+    pub fn with_decrease(mut self, decrease: Handle<UiNode>) -> Self {
         self.decrease = Some(decrease);
         self
     }
 
-    pub fn with_indicator(mut self, indicator: Handle<UINode<M, C>>) -> Self {
+    pub fn with_indicator(mut self, indicator: Handle<UiNode>) -> Self {
         self.indicator = Some(indicator);
         self
     }
 
-    pub fn with_body(mut self, body: Handle<UINode<M, C>>) -> Self {
+    pub fn with_body(mut self, body: Handle<UiNode>) -> Self {
         self.body = Some(body);
         self
     }
@@ -450,7 +444,7 @@ impl<M: MessageData, C: Control<M, C>> ScrollBarBuilder<M, C> {
         self
     }
 
-    pub fn build(self, ctx: &mut BuildContext<M, C>) -> Handle<UINode<M, C>> {
+    pub fn build(self, ctx: &mut BuildContext) -> Handle<UiNode> {
         let orientation = self.orientation.unwrap_or(Orientation::Horizontal);
 
         let increase = self.increase.unwrap_or_else(|| {
@@ -565,7 +559,7 @@ impl<M: MessageData, C: Control<M, C>> ScrollBarBuilder<M, C> {
 
         let field = CanvasBuilder::new(
             WidgetBuilder::new()
-                .with_name(ScrollBar::<M, C>::PART_CANVAS)
+                .with_name(ScrollBar::PART_CANVAS)
                 .on_column(match orientation {
                     Orientation::Horizontal => 1,
                     Orientation::Vertical => 0,
@@ -603,7 +597,7 @@ impl<M: MessageData, C: Control<M, C>> ScrollBarBuilder<M, C> {
         });
         ctx.link(grid, body);
 
-        let node = UINode::ScrollBar(ScrollBar {
+        let node = UiNode::new(ScrollBar {
             widget: self.widget_builder.with_child(body).build(),
             min,
             max,
