@@ -1,36 +1,49 @@
-use crate::core::algebra::Vector2;
 use crate::{
-    core::pool::Handle,
+    core::{algebra::Vector2, pool::Handle},
+    define_constructor,
     grid::{Column, GridBuilder, Row},
-    message::{
-        MessageData, MessageDirection, ScrollBarMessage, ScrollPanelMessage, ScrollViewerMessage,
-        UiMessage, UiMessageData, WidgetMessage,
-    },
-    scroll_bar::ScrollBarBuilder,
-    scroll_panel::ScrollPanelBuilder,
-    widget::{Widget, WidgetBuilder},
-    BuildContext, Control, NodeHandleMapping, Orientation, UINode, UserInterface,
+    message::{MessageDirection, UiMessage},
+    scroll_bar::{ScrollBar, ScrollBarBuilder, ScrollBarMessage},
+    scroll_panel::{ScrollPanelBuilder, ScrollPanelMessage},
+    widget::{Widget, WidgetBuilder, WidgetMessage},
+    BuildContext, Control, NodeHandleMapping, Orientation, UiNode, UserInterface,
+};
+use std::{
+    any::{Any, TypeId},
+    ops::{Deref, DerefMut},
 };
-use std::ops::{Deref, DerefMut};
 
-#[derive(Clone)]
-pub struct ScrollViewer<M: MessageData, C: Control<M, C>> {
-    pub widget: Widget<M, C>,
-    pub content: Handle<UINode<M, C>>,
-    pub scroll_panel: Handle<UINode<M, C>>,
-    pub v_scroll_bar: Handle<UINode<M, C>>,
-    pub h_scroll_bar: Handle<UINode<M, C>>,
+#[derive(Debug, Clone, PartialEq)]
+pub enum ScrollViewerMessage {
+    Content(Handle<UiNode>),
+    /// Adjusts vertical and horizontal scroll values so given node will be in "view box"
+    /// of scroll viewer.
+    BringIntoView(Handle<UiNode>),
 }
 
-crate::define_widget_deref!(ScrollViewer<M, C>);
+impl ScrollViewerMessage {
+    define_constructor!(ScrollViewerMessage:Content => fn content(Handle<UiNode>), layout: false);
+    define_constructor!(ScrollViewerMessage:BringIntoView=> fn bring_into_view(Handle<UiNode>), layout: true);
+}
 
-impl<M: MessageData, C: Control<M, C>> ScrollViewer<M, C> {
+#[derive(Clone)]
+pub struct ScrollViewer {
+    pub widget: Widget,
+    pub content: Handle<UiNode>,
+    pub scroll_panel: Handle<UiNode>,
+    pub v_scroll_bar: Handle<UiNode>,
+    pub h_scroll_bar: Handle<UiNode>,
+}
+
+crate::define_widget_deref!(ScrollViewer);
+
+impl ScrollViewer {
     pub fn new(
-        widget: Widget<M, C>,
-        content: Handle<UINode<M, C>>,
-        content_presenter: Handle<UINode<M, C>>,
-        v_scroll_bar: Handle<UINode<M, C>>,
-        h_scroll_bar: Handle<UINode<M, C>>,
+        widget: Widget,
+        content: Handle<UiNode>,
+        content_presenter: Handle<UiNode>,
+        v_scroll_bar: Handle<UiNode>,
+        h_scroll_bar: Handle<UiNode>,
     ) -> Self {
         Self {
             widget,
@@ -41,28 +54,36 @@ impl<M: MessageData, C: Control<M, C>> ScrollViewer<M, C> {
         }
     }
 
-    pub fn content_presenter(&self) -> Handle<UINode<M, C>> {
+    pub fn content_presenter(&self) -> Handle<UiNode> {
         self.scroll_panel
     }
 
-    pub fn content(&self) -> Handle<UINode<M, C>> {
+    pub fn content(&self) -> Handle<UiNode> {
         self.content
     }
 
-    pub fn set_content(&mut self, content: Handle<UINode<M, C>>) {
+    pub fn set_content(&mut self, content: Handle<UiNode>) {
         self.content = content;
     }
 }
 
-impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollViewer<M, C> {
-    fn resolve(&mut self, node_map: &NodeHandleMapping<M, C>) {
+impl Control for ScrollViewer {
+    fn query_component(&self, type_id: TypeId) -> Option<&dyn Any> {
+        if type_id == TypeId::of::<Self>() {
+            Some(self)
+        } else {
+            None
+        }
+    }
+
+    fn resolve(&mut self, node_map: &NodeHandleMapping) {
         node_map.resolve(&mut self.content);
         node_map.resolve(&mut self.scroll_panel);
         node_map.resolve(&mut self.v_scroll_bar);
         node_map.resolve(&mut self.h_scroll_bar);
     }
 
-    fn arrange_override(&self, ui: &UserInterface<M, C>, final_size: Vector2<f32>) -> Vector2<f32> {
+    fn arrange_override(&self, ui: &UserInterface, final_size: Vector2<f32>) -> Vector2<f32> {
         let size = self.widget.arrange_override(ui, final_size);
 
         if self.content.is_some() {
@@ -87,53 +108,45 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollViewer<M, C> {
         size
     }
 
-    fn handle_routed_message(
-        &mut self,
-        ui: &mut UserInterface<M, C>,
-        message: &mut UiMessage<M, C>,
-    ) {
+    fn handle_routed_message(&mut self, ui: &mut UserInterface, message: &mut UiMessage) {
         self.widget.handle_routed_message(ui, message);
 
-        match &message.data() {
-            UiMessageData::Widget(WidgetMessage::MouseWheel { amount, .. }) => {
-                if self.v_scroll_bar.is_some() && !message.handled() {
-                    if let UINode::ScrollBar(v_scroll_bar) = ui.node(self.v_scroll_bar) {
-                        let old_value = v_scroll_bar.value();
-                        let new_value = old_value - amount * 17.0;
-                        if (old_value - new_value).abs() > f32::EPSILON {
-                            message.set_handled(true);
-                        }
-                        ui.send_message(ScrollBarMessage::value(
-                            self.v_scroll_bar,
-                            MessageDirection::ToWidget,
-                            new_value,
-                        ));
+        if let Some(WidgetMessage::MouseWheel { amount, .. }) = message.data::<WidgetMessage>() {
+            if self.v_scroll_bar.is_some() && !message.handled() {
+                if let Some(v_scroll_bar) = ui.node(self.v_scroll_bar).cast::<ScrollBar>() {
+                    let old_value = v_scroll_bar.value();
+                    let new_value = old_value - amount * 17.0;
+                    if (old_value - new_value).abs() > f32::EPSILON {
+                        message.set_handled(true);
                     }
+                    ui.send_message(ScrollBarMessage::value(
+                        self.v_scroll_bar,
+                        MessageDirection::ToWidget,
+                        new_value,
+                    ));
                 }
             }
-            UiMessageData::ScrollPanel(msg) => {
-                if message.destination() == self.scroll_panel {
-                    let msg = match *msg {
-                        ScrollPanelMessage::VerticalScroll(value) => ScrollBarMessage::value(
-                            self.v_scroll_bar,
-                            MessageDirection::ToWidget,
-                            value,
-                        ),
-                        ScrollPanelMessage::HorizontalScroll(value) => ScrollBarMessage::value(
-                            self.h_scroll_bar,
-                            MessageDirection::ToWidget,
-                            value,
-                        ),
-                        _ => return,
-                    };
-                    // handle flag here is raised to prevent infinite message loop with the branch down below (ScrollBar::value).
-                    msg.set_handled(true);
-                    ui.send_message(msg);
-                }
+        } else if let Some(msg) = message.data::<ScrollPanelMessage>() {
+            if message.destination() == self.scroll_panel {
+                let msg = match *msg {
+                    ScrollPanelMessage::VerticalScroll(value) => ScrollBarMessage::value(
+                        self.v_scroll_bar,
+                        MessageDirection::ToWidget,
+                        value,
+                    ),
+                    ScrollPanelMessage::HorizontalScroll(value) => ScrollBarMessage::value(
+                        self.h_scroll_bar,
+                        MessageDirection::ToWidget,
+                        value,
+                    ),
+                    _ => return,
+                };
+                // handle flag here is raised to prevent infinite message loop with the branch down below (ScrollBar::value).
+                msg.set_handled(true);
+                ui.send_message(msg);
             }
-            UiMessageData::ScrollBar(msg)
-                if message.direction() == MessageDirection::FromWidget =>
-            {
+        } else if let Some(msg) = message.data::<ScrollBarMessage>() {
+            if message.direction() == MessageDirection::FromWidget {
                 match msg {
                     ScrollBarMessage::Value(new_value) => {
                         if !message.handled() {
@@ -159,7 +172,8 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollViewer<M, C> {
                     &ScrollBarMessage::MaxValue(_) => {
                         if message.destination() == self.v_scroll_bar && self.v_scroll_bar.is_some()
                         {
-                            if let UINode::ScrollBar(scroll_bar) = ui.node(self.v_scroll_bar) {
+                            if let Some(scroll_bar) = ui.node(self.v_scroll_bar).cast::<ScrollBar>()
+                            {
                                 let visibility = (scroll_bar.max_value() - scroll_bar.min_value())
                                     .abs()
                                     >= f32::EPSILON;
@@ -172,7 +186,8 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollViewer<M, C> {
                         } else if message.destination() == self.h_scroll_bar
                             && self.h_scroll_bar.is_some()
                         {
-                            if let UINode::ScrollBar(scroll_bar) = ui.node(self.h_scroll_bar) {
+                            if let Some(scroll_bar) = ui.node(self.h_scroll_bar).cast::<ScrollBar>()
+                            {
                                 let visibility = (scroll_bar.max_value() - scroll_bar.min_value())
                                     .abs()
                                     >= f32::EPSILON;
@@ -187,64 +202,47 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for ScrollViewer<M, C> {
                     _ => (),
                 }
             }
-            UiMessageData::ScrollViewer(msg) => {
-                if message.destination() == self.handle() {
-                    match msg {
-                        ScrollViewerMessage::Content(content) => {
-                            for child in ui.node(self.scroll_panel).children().to_vec() {
-                                ui.send_message(WidgetMessage::remove(
-                                    child,
-                                    MessageDirection::ToWidget,
-                                ));
-                            }
-                            ui.send_message(WidgetMessage::link(
-                                *content,
+        } else if let Some(msg) = message.data::<ScrollViewerMessage>() {
+            if message.destination() == self.handle() {
+                match msg {
+                    ScrollViewerMessage::Content(content) => {
+                        for child in ui.node(self.scroll_panel).children().to_vec() {
+                            ui.send_message(WidgetMessage::remove(
+                                child,
                                 MessageDirection::ToWidget,
-                                self.scroll_panel,
-                            ));
-                        }
-                        &ScrollViewerMessage::BringIntoView(handle) => {
-                            // Re-cast message to inner panel.
-                            ui.send_message(ScrollPanelMessage::bring_into_view(
-                                self.scroll_panel,
-                                MessageDirection::ToWidget,
-                                handle,
                             ));
                         }
+                        ui.send_message(WidgetMessage::link(
+                            *content,
+                            MessageDirection::ToWidget,
+                            self.scroll_panel,
+                        ));
+                    }
+                    &ScrollViewerMessage::BringIntoView(handle) => {
+                        // Re-cast message to inner panel.
+                        ui.send_message(ScrollPanelMessage::bring_into_view(
+                            self.scroll_panel,
+                            MessageDirection::ToWidget,
+                            handle,
+                        ));
                     }
                 }
             }
-            _ => {}
-        }
-    }
-
-    fn remove_ref(&mut self, handle: Handle<UINode<M, C>>) {
-        if self.content == handle {
-            self.content = Handle::NONE;
-        }
-        if self.v_scroll_bar == handle {
-            self.v_scroll_bar = Handle::NONE;
-        }
-        if self.h_scroll_bar == handle {
-            self.h_scroll_bar = Handle::NONE;
-        }
-        if self.scroll_panel == handle {
-            self.scroll_panel = Handle::NONE;
         }
     }
 }
 
-pub struct ScrollViewerBuilder<M: MessageData, C: Control<M, C>> {
-    widget_builder: WidgetBuilder<M, C>,
-    content: Handle<UINode<M, C>>,
-    h_scroll_bar: Option<Handle<UINode<M, C>>>,
-    v_scroll_bar: Option<Handle<UINode<M, C>>>,
+pub struct ScrollViewerBuilder {
+    widget_builder: WidgetBuilder,
+    content: Handle<UiNode>,
+    h_scroll_bar: Option<Handle<UiNode>>,
+    v_scroll_bar: Option<Handle<UiNode>>,
     horizontal_scroll_allowed: bool,
     vertical_scroll_allowed: bool,
 }
 
-impl<M: MessageData, C: Control<M, C>> ScrollViewerBuilder<M, C> {
-    pub fn new(widget_builder: WidgetBuilder<M, C>) -> Self {
+impl ScrollViewerBuilder {
+    pub fn new(widget_builder: WidgetBuilder) -> Self {
         Self {
             widget_builder,
             content: Handle::NONE,
@@ -255,17 +253,17 @@ impl<M: MessageData, C: Control<M, C>> ScrollViewerBuilder<M, C> {
         }
     }
 
-    pub fn with_content(mut self, content: Handle<UINode<M, C>>) -> Self {
+    pub fn with_content(mut self, content: Handle<UiNode>) -> Self {
         self.content = content;
         self
     }
 
-    pub fn with_vertical_scroll_bar(mut self, v_scroll_bar: Handle<UINode<M, C>>) -> Self {
+    pub fn with_vertical_scroll_bar(mut self, v_scroll_bar: Handle<UiNode>) -> Self {
         self.v_scroll_bar = Some(v_scroll_bar);
         self
     }
 
-    pub fn with_horizontal_scroll_bar(mut self, h_scroll_bar: Handle<UINode<M, C>>) -> Self {
+    pub fn with_horizontal_scroll_bar(mut self, h_scroll_bar: Handle<UiNode>) -> Self {
         self.h_scroll_bar = Some(h_scroll_bar);
         self
     }
@@ -280,7 +278,7 @@ impl<M: MessageData, C: Control<M, C>> ScrollViewerBuilder<M, C> {
         self
     }
 
-    pub fn build(self, ctx: &mut BuildContext<M, C>) -> Handle<UINode<M, C>> {
+    pub fn build(self, ctx: &mut BuildContext) -> Handle<UiNode> {
         let content_presenter = ScrollPanelBuilder::new(
             WidgetBuilder::new()
                 .with_child(self.content)
@@ -327,6 +325,6 @@ impl<M: MessageData, C: Control<M, C>> ScrollViewerBuilder<M, C> {
             h_scroll_bar,
             scroll_panel: content_presenter,
         };
-        ctx.add_node(UINode::ScrollViewer(sv))
+        ctx.add_node(UiNode::new(sv))
     }
 }
