@@ -1,36 +1,55 @@
-use crate::core::algebra::Vector2;
-use crate::formatted_text::WrapMode;
-use crate::message::MessageData;
-use crate::ttf::SharedFont;
 use crate::{
     brush::Brush,
-    core::{color::Color, pool::Handle},
+    core::{algebra::Vector2, color::Color, pool::Handle},
+    define_constructor,
     draw::DrawingContext,
-    formatted_text::{FormattedText, FormattedTextBuilder},
-    message::UiMessage,
-    message::{TextMessage, UiMessageData},
+    formatted_text::{FormattedText, FormattedTextBuilder, WrapMode},
+    message::{MessageDirection, UiMessage},
+    ttf::SharedFont,
     widget::{Widget, WidgetBuilder},
-    BuildContext, Control, HorizontalAlignment, UINode, UserInterface, VerticalAlignment,
+    BuildContext, Control, HorizontalAlignment, UiNode, UserInterface, VerticalAlignment,
 };
 use std::{
+    any::{Any, TypeId},
     cell::RefCell,
     ops::{Deref, DerefMut},
 };
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum TextMessage {
+    Text(String),
+    Wrap(WrapMode),
+    Font(SharedFont),
+    VerticalAlignment(VerticalAlignment),
+    HorizontalAlignment(HorizontalAlignment),
+}
+
+impl TextMessage {
+    define_constructor!(TextMessage:Text => fn text(String), layout: false);
+    define_constructor!(TextMessage:Wrap=> fn wrap(WrapMode), layout: false);
+    define_constructor!(TextMessage:Font => fn font(SharedFont), layout: false);
+    define_constructor!(TextMessage:VerticalAlignment => fn vertical_alignment(VerticalAlignment), layout: false);
+    define_constructor!(TextMessage:HorizontalAlignment => fn horizontal_alignment(HorizontalAlignment), layout: false);
+}
+
 #[derive(Clone)]
-pub struct Text<M: MessageData, C: Control<M, C>> {
-    widget: Widget<M, C>,
+pub struct Text {
+    widget: Widget,
     formatted_text: RefCell<FormattedText>,
 }
 
-crate::define_widget_deref!(Text<M, C>);
+crate::define_widget_deref!(Text);
 
-impl<M: MessageData, C: Control<M, C>> Control<M, C> for Text<M, C> {
-    fn measure_override(
-        &self,
-        _: &UserInterface<M, C>,
-        available_size: Vector2<f32>,
-    ) -> Vector2<f32> {
+impl Control for Text {
+    fn query_component(&self, type_id: TypeId) -> Option<&dyn Any> {
+        if type_id == TypeId::of::<Self>() {
+            Some(self)
+        } else {
+            None
+        }
+    }
+
+    fn measure_override(&self, _: &UserInterface, available_size: Vector2<f32>) -> Vector2<f32> {
         self.formatted_text
             .borrow_mut()
             .set_constraint(available_size)
@@ -47,15 +66,11 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for Text<M, C> {
         );
     }
 
-    fn handle_routed_message(
-        &mut self,
-        ui: &mut UserInterface<M, C>,
-        message: &mut UiMessage<M, C>,
-    ) {
+    fn handle_routed_message(&mut self, ui: &mut UserInterface, message: &mut UiMessage) {
         self.widget.handle_routed_message(ui, message);
 
         if message.destination() == self.handle() {
-            if let UiMessageData::Text(msg) = &message.data() {
+            if let Some(msg) = message.data::<TextMessage>() {
                 match msg {
                     TextMessage::Text(text) => {
                         self.formatted_text.borrow_mut().set_text(text);
@@ -89,8 +104,8 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for Text<M, C> {
     }
 }
 
-impl<M: MessageData, C: Control<M, C>> Text<M, C> {
-    pub fn new(widget: Widget<M, C>) -> Self {
+impl Text {
+    pub fn new(widget: Widget) -> Self {
         Self {
             widget,
             formatted_text: RefCell::new(
@@ -110,7 +125,7 @@ impl<M: MessageData, C: Control<M, C>> Text<M, C> {
     }
 
     pub fn font(&self) -> SharedFont {
-        self.formatted_text.borrow().get_font().unwrap()
+        self.formatted_text.borrow().get_font()
     }
 
     pub fn vertical_alignment(&self) -> VerticalAlignment {
@@ -122,8 +137,8 @@ impl<M: MessageData, C: Control<M, C>> Text<M, C> {
     }
 }
 
-pub struct TextBuilder<M: MessageData, C: Control<M, C>> {
-    widget_builder: WidgetBuilder<M, C>,
+pub struct TextBuilder {
+    widget_builder: WidgetBuilder,
     text: Option<String>,
     font: Option<SharedFont>,
     vertical_text_alignment: VerticalAlignment,
@@ -131,8 +146,8 @@ pub struct TextBuilder<M: MessageData, C: Control<M, C>> {
     wrap: WrapMode,
 }
 
-impl<M: MessageData, C: Control<M, C>> TextBuilder<M, C> {
-    pub fn new(widget_builder: WidgetBuilder<M, C>) -> Self {
+impl TextBuilder {
+    pub fn new(widget_builder: WidgetBuilder) -> Self {
         Self {
             widget_builder,
             text: None,
@@ -173,7 +188,7 @@ impl<M: MessageData, C: Control<M, C>> TextBuilder<M, C> {
         self
     }
 
-    pub fn build(mut self, ui: &mut BuildContext<M, C>) -> Handle<UINode<M, C>> {
+    pub fn build(mut self, ui: &mut BuildContext) -> Handle<UiNode> {
         let font = if let Some(font) = self.font {
             font
         } else {
@@ -196,6 +211,6 @@ impl<M: MessageData, C: Control<M, C>> TextBuilder<M, C> {
                     .build(),
             ),
         };
-        ui.add_node(UINode::Text(text))
+        ui.add_node(UiNode::new(text))
     }
 }
