@@ -1,28 +1,35 @@
-use crate::draw::Draw;
-use crate::formatted_text::WrapMode;
 use crate::{
     brush::Brush,
     core::{algebra::Vector2, color::Color, math::Rect, pool::Handle},
-    draw::{CommandTexture, DrawingContext},
-    formatted_text::{FormattedText, FormattedTextBuilder},
-    message::{
-        CursorIcon, KeyCode, MessageData, MessageDirection, MouseButton, TextBoxMessage, UiMessage,
-        UiMessageData, WidgetMessage,
-    },
+    define_constructor,
+    draw::{CommandTexture, Draw, DrawingContext},
+    formatted_text::{FormattedText, FormattedTextBuilder, WrapMode},
+    message::{CursorIcon, KeyCode, MessageDirection, MouseButton, UiMessage},
     ttf::SharedFont,
-    widget::{Widget, WidgetBuilder},
-    BuildContext, Control, HorizontalAlignment, UINode, UserInterface, VerticalAlignment,
+    widget::{Widget, WidgetBuilder, WidgetMessage},
+    BuildContext, Control, HorizontalAlignment, UiNode, UserInterface, VerticalAlignment,
     BRUSH_DARKER, BRUSH_TEXT,
 };
 use copypasta::ClipboardProvider;
 use std::{
+    any::{Any, TypeId},
     cell::RefCell,
     cmp::{self, Ordering},
     fmt::{Debug, Formatter},
     ops::{Deref, DerefMut},
     rc::Rc,
+    sync::mpsc::Sender,
 };
 
+#[derive(Debug, Clone, PartialEq)]
+pub enum TextBoxMessage {
+    Text(String),
+}
+
+impl TextBoxMessage {
+    define_constructor!(TextBoxMessage:Text => fn text(String), layout: false);
+}
+
 #[derive(Copy, Clone, PartialEq, Eq)]
 pub enum HorizontalDirection {
     Left,
@@ -94,8 +101,8 @@ impl SelectionRange {
 pub type FilterCallback = dyn FnMut(char) -> bool;
 
 #[derive(Clone)]
-pub struct TextBox<M: MessageData, C: Control<M, C>> {
-    widget: Widget<M, C>,
+pub struct TextBox {
+    widget: Widget,
     caret_position: Position,
     caret_visible: bool,
     blink_timer: f32,
@@ -112,15 +119,15 @@ pub struct TextBox<M: MessageData, C: Control<M, C>> {
     editable: bool,
 }
 
-impl<M: MessageData, C: Control<M, C>> Debug for TextBox<M, C> {
+impl Debug for TextBox {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         f.write_str("TextBox")
     }
 }
 
-crate::define_widget_deref!(TextBox<M, C>);
+crate::define_widget_deref!(TextBox);
 
-impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
+impl TextBox {
     pub fn reset_blink(&mut self) {
         self.caret_visible = true;
         self.blink_timer = 0.0;
@@ -243,7 +250,7 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
     }
 
     /// Inserts given character at current caret position.
-    fn insert_char(&mut self, c: char, ui: &UserInterface<M, C>) {
+    fn insert_char(&mut self, c: char, ui: &UserInterface) {
         if !c.is_control() {
             let position = self.get_absolute_position(self.caret_position).unwrap_or(0);
             self.formatted_text
@@ -259,7 +266,7 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
         }
     }
 
-    fn insert_str(&mut self, str: &str, ui: &UserInterface<M, C>) {
+    fn insert_str(&mut self, str: &str, ui: &UserInterface) {
         let position = self.get_absolute_position(self.caret_position).unwrap_or(0);
         self.formatted_text.borrow_mut().insert_str(str, position);
         self.move_caret_x(str.chars().count(), HorizontalDirection::Right, false);
@@ -274,7 +281,7 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
         self.formatted_text.borrow_mut().get_raw_text().len()
     }
 
-    fn remove_char(&mut self, direction: HorizontalDirection, ui: &UserInterface<M, C>) {
+    fn remove_char(&mut self, direction: HorizontalDirection, ui: &UserInterface) {
         if let Some(position) = self.get_absolute_position(self.caret_position) {
             let text_len = self.get_text_len();
             if text_len != 0 {
@@ -308,7 +315,7 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
         }
     }
 
-    fn remove_range(&mut self, ui: &UserInterface<M, C>, selection: SelectionRange) {
+    fn remove_range(&mut self, ui: &UserInterface, selection: SelectionRange) {
         let selection = selection.normalized();
         if let Some(begin) = self.get_absolute_position(selection.begin) {
             if let Some(end) = self.get_absolute_position(selection.end) {
@@ -328,21 +335,22 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
 
     pub fn screen_pos_to_text_pos(&self, screen_pos: Vector2<f32>) -> Option<Position> {
         let caret_pos = self.widget.screen_position;
-        if let Some(font) = self.formatted_text.borrow().get_font() {
-            let font = font.0.lock().unwrap();
-            for (line_index, line) in self.formatted_text.borrow().get_lines().iter().enumerate() {
-                let line_bounds = Rect::new(
-                    caret_pos.x + line.x_offset,
-                    caret_pos.y + line.y_offset,
-                    line.width,
-                    font.ascender(),
-                );
-                if line_bounds.contains(screen_pos) {
-                    let mut x = line_bounds.x();
-                    // Check each character in line.
-                    for (offset, index) in (line.begin..line.end).enumerate() {
-                        let symbol = self.formatted_text.borrow().get_raw_text()[index];
-                        let (width, height, advance) = if let Some(glyph) = font.glyph(symbol) {
+        let font = self.formatted_text.borrow().get_font();
+        let font = font.0.lock().unwrap();
+        for (line_index, line) in self.formatted_text.borrow().get_lines().iter().enumerate() {
+            let line_bounds = Rect::new(
+                caret_pos.x + line.x_offset,
+                caret_pos.y + line.y_offset,
+                line.width,
+                font.ascender(),
+            );
+            if line_bounds.contains(screen_pos) {
+                let mut x = line_bounds.x();
+                // Check each character in line.
+                for (offset, index) in (line.begin..line.end).enumerate() {
+                    let character = self.formatted_text.borrow().get_raw_text()[index];
+                    let (width, height, advance) =
+                        if let Some(glyph) = font.glyphs().get(character.char_code as usize) {
                             (
                                 glyph.bitmap_width as f32,
                                 glyph.bitmap_height as f32,
@@ -353,15 +361,14 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
                             let h = font.height();
                             (h, h, h)
                         };
-                        let char_bounds = Rect::new(x, line_bounds.y(), width, height);
-                        if char_bounds.contains(screen_pos) {
-                            return Some(Position {
-                                line: line_index,
-                                offset,
-                            });
-                        }
-                        x += advance;
+                    let char_bounds = Rect::new(x, line_bounds.y(), width, height);
+                    if char_bounds.contains(screen_pos) {
+                        return Some(Position {
+                            line: line_index,
+                            offset,
+                        });
                     }
+                    x += advance;
                 }
             }
         }
@@ -387,7 +394,7 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
     }
 
     pub fn font(&self) -> SharedFont {
-        self.formatted_text.borrow().get_font().unwrap()
+        self.formatted_text.borrow().get_font()
     }
 
     pub fn set_vertical_alignment(&mut self, valign: VerticalAlignment) -> &mut Self {
@@ -413,12 +420,16 @@ impl<M: MessageData, C: Control<M, C>> TextBox<M, C> {
     }
 }
 
-impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
-    fn measure_override(
-        &self,
-        _: &UserInterface<M, C>,
-        available_size: Vector2<f32>,
-    ) -> Vector2<f32> {
+impl Control for TextBox {
+    fn query_component(&self, type_id: TypeId) -> Option<&dyn Any> {
+        if type_id == TypeId::of::<Self>() {
+            Some(self)
+        } else {
+            None
+        }
+    }
+
+    fn measure_override(&self, _: &UserInterface, available_size: Vector2<f32>) -> Vector2<f32> {
         self.formatted_text
             .borrow_mut()
             .set_constraint(available_size)
@@ -515,38 +526,37 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
         if self.caret_visible {
             let text = self.formatted_text.borrow();
 
-            if let Some(font) = text.get_font() {
-                let mut caret_pos = screen_position;
+            let font = text.get_font();
+            let mut caret_pos = screen_position;
 
-                let font = font.0.lock().unwrap();
-                if let Some(line) = text.get_lines().get(self.caret_position.line) {
-                    let text = text.get_raw_text();
-                    caret_pos += Vector2::new(line.x_offset, line.y_offset);
-                    for (offset, char_index) in (line.begin..line.end).enumerate() {
-                        if offset >= self.caret_position.offset {
-                            break;
-                        }
-                        if let Some(glyph) = font.glyph(text[char_index]) {
-                            caret_pos.x += glyph.advance;
-                        } else {
-                            caret_pos.x += font.height();
-                        }
+            let font = font.0.lock().unwrap();
+            if let Some(line) = text.get_lines().get(self.caret_position.line) {
+                let text = text.get_raw_text();
+                caret_pos += Vector2::new(line.x_offset, line.y_offset);
+                for (offset, char_index) in (line.begin..line.end).enumerate() {
+                    if offset >= self.caret_position.offset {
+                        break;
+                    }
+                    if let Some(glyph) = font.glyphs().get(text[char_index].glyph_index as usize) {
+                        caret_pos.x += glyph.advance;
+                    } else {
+                        caret_pos.x += font.height();
                     }
                 }
-
-                let caret_bounds = Rect::new(caret_pos.x, caret_pos.y, 2.0, font.height());
-                drawing_context.push_rect_filled(&caret_bounds, None);
-                drawing_context.commit(
-                    self.clip_bounds(),
-                    self.caret_brush.clone(),
-                    CommandTexture::None,
-                    None,
-                );
             }
+
+            let caret_bounds = Rect::new(caret_pos.x, caret_pos.y, 2.0, font.height());
+            drawing_context.push_rect_filled(&caret_bounds, None);
+            drawing_context.commit(
+                self.clip_bounds(),
+                self.caret_brush.clone(),
+                CommandTexture::None,
+                None,
+            );
         }
     }
 
-    fn update(&mut self, dt: f32) {
+    fn update(&mut self, dt: f32, _sender: &Sender<UiMessage>) {
         if self.has_focus {
             self.blink_timer += dt;
             if self.blink_timer >= self.blink_interval {
@@ -558,16 +568,12 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
         }
     }
 
-    fn handle_routed_message(
-        &mut self,
-        ui: &mut UserInterface<M, C>,
-        message: &mut UiMessage<M, C>,
-    ) {
+    fn handle_routed_message(&mut self, ui: &mut UserInterface, message: &mut UiMessage) {
         self.widget.handle_routed_message(ui, message);
 
         if message.destination() == self.handle() {
-            match &message.data() {
-                UiMessageData::Widget(msg) => match msg {
+            if let Some(msg) = message.data::<WidgetMessage>() {
+                match msg {
                     &WidgetMessage::Text(symbol)
                         if !ui.keyboard_modifiers().control
                             && !ui.keyboard_modifiers().alt
@@ -786,19 +792,18 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
                         ui.release_mouse_capture();
                     }
                     _ => {}
-                },
-                UiMessageData::TextBox(TextBoxMessage::Text(new_text))
-                    if message.direction() == MessageDirection::ToWidget =>
-                {
+                }
+            } else if let Some(TextBoxMessage::Text(new_text)) = message.data::<TextBoxMessage>() {
+                if message.direction() == MessageDirection::ToWidget {
                     let mut equals = false;
-                    for (&new, old) in self
+                    for (&old, new) in self
                         .formatted_text
                         .borrow()
                         .get_raw_text()
                         .iter()
                         .zip(new_text.chars())
                     {
-                        if old as u32 != new {
+                        if old.char_code != new as u32 {
                             equals = false;
                             break;
                         }
@@ -812,14 +817,13 @@ impl<M: MessageData, C: Control<M, C>> Control<M, C> for TextBox<M, C> {
                         }
                     }
                 }
-                _ => {}
             }
         }
     }
 }
 
-pub struct TextBoxBuilder<M: MessageData, C: Control<M, C>> {
-    widget_builder: WidgetBuilder<M, C>,
+pub struct TextBoxBuilder {
+    widget_builder: WidgetBuilder,
     font: Option<SharedFont>,
     text: String,
     caret_brush: Brush,
@@ -834,8 +838,8 @@ pub struct TextBoxBuilder<M: MessageData, C: Control<M, C>> {
     mask_char: Option<char>,
 }
 
-impl<M: MessageData, C: Control<M, C>> TextBoxBuilder<M, C> {
-    pub fn new(widget_builder: WidgetBuilder<M, C>) -> Self {
+impl TextBoxBuilder {
+    pub fn new(widget_builder: WidgetBuilder) -> Self {
         Self {
             widget_builder,
             font: None,
@@ -913,7 +917,7 @@ impl<M: MessageData, C: Control<M, C>> TextBoxBuilder<M, C> {
         self
     }
 
-    pub fn build(mut self, ctx: &mut BuildContext<M, C>) -> Handle<UINode<M, C>> {
+    pub fn build(mut self, ctx: &mut BuildContext) -> Handle<UiNode> {
         if self.widget_builder.foreground.is_none() {
             self.widget_builder.foreground = Some(BRUSH_TEXT);
         }
@@ -951,6 +955,6 @@ impl<M: MessageData, C: Control<M, C>> TextBoxBuilder<M, C> {
             editable: self.editable,
         };
 
-        ctx.add_node(UINode::TextBox(text_box))
+        ctx.add_node(UiNode::new(text_box))
     }
 }
