@@ -1,20 +1,314 @@
 use crate::{
     brush::Brush,
     core::{algebra::Vector2, math::Rect, pool::Handle},
-    message::{CursorIcon, MessageData, MessageDirection, UiMessage, UiMessageData, WidgetMessage},
-    Control, HorizontalAlignment, Thickness, UINode, UserInterface, VerticalAlignment,
-    BRUSH_FOREGROUND, BRUSH_PRIMARY,
+    define_constructor,
+    message::{CursorIcon, KeyCode, MessageDirection, UiMessage},
+    HorizontalAlignment, LayoutEvent, MouseButton, MouseState, Thickness, UiNode, UserInterface,
+    VerticalAlignment, BRUSH_FOREGROUND, BRUSH_PRIMARY,
 };
 use std::{
     any::Any,
     cell::{Cell, RefCell},
-    marker::PhantomData,
     rc::Rc,
+    sync::mpsc::Sender,
 };
 
+/// A set of messages for any kind of widgets (including user controls). These messages provides basic
+/// communication elements of the UI library.
+#[derive(Debug, Clone, PartialEq)]
+pub enum WidgetMessage {
+    /// Initiated when user clicks on a widget's geometry.
+    ///
+    /// Direction: **From UI**.
+    MouseDown {
+        /// Position of cursor.
+        pos: Vector2<f32>,
+        /// A button that was pressed.
+        button: MouseButton,
+    },
+
+    /// Initiated when user releases mouse button while cursor is over widget's geometry.
+    ///
+    /// Direction: **From UI**.
+    MouseUp {
+        /// Position of cursor.
+        pos: Vector2<f32>,
+        /// A button that was released.
+        button: MouseButton,
+    },
+
+    /// Initiated when user moves cursor over widget's geometry.
+    ///
+    /// Direction: **From/To UI**.
+    MouseMove {
+        /// New position of cursor in screen coordinates.
+        pos: Vector2<f32>,
+        /// State of mouse buttons.
+        state: MouseState,
+    },
+
+    /// Initiated when user scrolls mouse wheel while cursor is over widget's geometry.
+    ///
+    /// Direction: **From/To UI**.
+    MouseWheel {
+        /// Position of cursor.
+        pos: Vector2<f32>,
+        /// Amount of lines per mouse wheel turn.
+        amount: f32,
+    },
+
+    /// Initiated when cursor leaves geometry of a widget.
+    ///
+    /// Direction: **From UI**.
+    MouseLeave,
+
+    /// Initiated when cursor enters geometry of a widget.
+    ///
+    /// Direction: **From UI**.
+    MouseEnter,
+
+    /// Initiated when widget is in focus and user types something on a keyboard.
+    ///
+    /// Direction: **From/To UI**.
+    Text(char),
+
+    /// Initiated when widget is in focus and user presses a button on a keyboard.
+    ///
+    /// Direction: **From UI**.
+    KeyDown(KeyCode),
+
+    /// Initiated when widget is in focus and user releases a button on a keyboard.
+    ///
+    /// Direction: **From UI**.
+    KeyUp(KeyCode),
+
+    /// Initiated when widget received focus. In most cases focus is received by clicking on
+    /// widget.
+    ///
+    /// Direction: **From UI**.
+    GotFocus,
+
+    /// Initiated when dragging of a widget has started.
+    ///
+    /// Direction: **From UI**.
+    DragStarted(Handle<UiNode>),
+
+    /// Initiated when user drags a widget over some other widget.
+    ///
+    /// Direction: **From UI**.
+    DragOver(Handle<UiNode>),
+
+    /// Initiated when user drops a widget onto some other widget.
+    ///
+    /// Direction: **From UI**.
+    Drop(Handle<UiNode>),
+
+    /// Initiated when widget has lost its focus.
+    ///
+    /// Direction: **From UI**.
+    LostFocus,
+
+    /// A request to make widget topmost. Widget can be made topmost only in the same hierarchy
+    /// level only!
+    ///
+    /// Direction: **From/To UI**.
+    TopMost,
+
+    /// A request to detach widget from its current parent and attach to root canvas.
+    ///
+    /// Direction: **From/To UI**.
+    Unlink,
+
+    /// A request to delete widget with all its children widgets. All handles to a node and its
+    /// children will be invalid after processing such message!
+    ///
+    /// Direction: **From/To UI**.
+    Remove,
+
+    /// A request to link initiator with specified widget.
+    ///
+    /// Direction: **From/To UI**.
+    LinkWith(Handle<UiNode>),
+
+    /// A request to link initiator with specified widget and put it in front of children list.
+    ///
+    /// Direction: **From/To UI**.
+    LinkWithReverse(Handle<UiNode>),
+
+    /// A request to change background brush of a widget. Background brushes are used to fill volume of widgets.
+    ///
+    /// Direction: **From/To UI**
+    Background(Brush),
+
+    /// A request to change foreground brush of a widget. Foreground brushes are used for text, borders and so on.
+    ///
+    /// Direction: **From/To UI**
+    Foreground(Brush),
+
+    /// A request to change name of a widget. Name is given to widget mostly for debugging purposes.
+    ///
+    /// Direction: **From/To UI**
+    Name(String),
+
+    /// A request to set width of a widget. In most cases there is no need to explicitly set width of a widget,
+    /// because rg3d-ui uses automatic layout engine which will correctly calculate desired width of a widget.
+    ///
+    /// Direction: **From/To UI**
+    Width(f32),
+
+    /// A request to set height of a widget. In most cases there is no need to explicitly set height of a widget,
+    /// because rg3d-ui uses automatic layout engine which will correctly calculate desired height of a widget.
+    ///
+    /// Direction: **From/To UI**
+    Height(f32),
+
+    /// A request to set vertical alignment of a widget. Vertical alignment tells where to put widget in the parent
+    /// widget's bounds in vertical direction.
+    ///
+    /// Direction: **From/To UI**
+    VerticalAlignment(VerticalAlignment),
+
+    /// A request to set horizontal alignment of a widget. Horizontal alignment tells where to put widget in the parent
+    /// widget's bounds in horizontal direction.
+    ///
+    /// Direction: **From/To UI**
+    HorizontalAlignment(HorizontalAlignment),
+
+    /// A request to set maximum size of widget. Maximum size restricts size of a widget during layout pass. For example
+    /// you can set maximum size to a button which was placed into a grid's cell, if maximum size wouldn't be set, button
+    /// would be stretched to fill entire cell.
+    ///
+    /// Direction: **From/To UI**
+    MaxSize(Vector2<f32>),
+
+    /// A request to set minimum size of widget. Minimum size restricts size of a widget during layout pass. For example
+    /// you can set minimum size to a button which was placed into a grid's cell, if minimum size wouldn't be set, button
+    /// would be compressed to fill entire cell.
+    ///
+    /// Direction: **From/To UI**
+    MinSize(Vector2<f32>),
+
+    /// A request to set row number of a grid to which widget should belong to.
+    ///
+    /// Direction: **From/To UI**
+    ///
+    /// # Notes
+    ///
+    /// This is bad API and it should be changed in future. Grid should have explicit list of pairs (row, child) instead
+    /// of this indirect attachment.
+    Row(usize),
+
+    /// A request to set column number of a grid to which widget should belong to.
+    ///
+    /// Direction: **From/To UI**
+    ///
+    /// # Notes
+    ///
+    /// This is bad API and it should be changed in future. Grid should have explicit list of pairs (column, child) instead
+    /// of this indirect attachment.
+    Column(usize),
+
+    /// A request to set new margin of widget. Margin could be used to add some free space around widget to make UI look less
+    /// dense.
+    ///
+    /// Direction: **From/To UI**
+    Margin(Thickness),
+
+    /// A request to set new state hit test visibility. If set to false, widget will become "non-clickable". It is useful for
+    /// decorations which should be transparent for mouse events.
+    ///
+    /// Direction: **From/To UI**
+    HitTestVisibility(bool),
+
+    /// A request to set new visibility of a widget. Widget can be either visible or not. Invisible widgets does not take space
+    /// in layout pass and collapsed to a point.
+    ///
+    /// Direction: **From/To UI**
+    Visibility(bool),
+
+    /// A request to set new z index of a widget. Z index is used to change drawing order of widgets. Please note that it works
+    /// only in same hierarchy level, which means that it is impossible to set z index to 9999 (or similar huge value) to force
+    /// widget to be drawn on top of everything.
+    ///
+    /// Direction: **From/To UI**
+    ZIndex(usize),
+
+    /// A request to set new desired position of a widget. It is called "desired" because layout system may ignore it and set
+    /// some other position. Desired position works with a combination of a layout panel that supports direct coordinated
+    /// (Canvas for example).
+    ///
+    /// Direction: **From/To UI**
+    DesiredPosition(Vector2<f32>),
+
+    /// A request to enable or disable widget. Disabled widget won't receive mouse events and may look differently (it is defined
+    /// by internal styling).
+    ///
+    /// Direction: **From/To UI**
+    Enabled(bool),
+
+    /// A request to set desired position at center in local coordinates.
+    ///
+    /// Direction: **From/To UI**
+    Center,
+
+    /// A request to set new cursor icon for widget.
+    ///
+    /// Direction: **From/To UI**
+    Cursor(Option<CursorIcon>),
+
+    /// A request to set new opacity for widget.
+    ///
+    /// Direction: **From/To UI**
+    Opacity(Option<f32>),
+}
+
+impl WidgetMessage {
+    define_constructor!(WidgetMessage:Remove => fn remove(), layout: false);
+    define_constructor!(WidgetMessage:Unlink => fn unlink(), layout: false);
+    define_constructor!(WidgetMessage:LinkWith => fn link(Handle<UiNode>), layout: false);
+    define_constructor!(WidgetMessage:LinkWithReverse => fn link_reverse(Handle<UiNode>), layout: false);
+    define_constructor!(WidgetMessage:Background => fn background(Brush), layout: false);
+    define_constructor!(WidgetMessage:Foreground => fn foreground(Brush), layout: false);
+    define_constructor!(WidgetMessage:Visibility => fn visibility(bool), layout: false);
+    define_constructor!(WidgetMessage:Width => fn width(f32), layout: false);
+    define_constructor!(WidgetMessage:Height => fn height(f32), layout: false);
+    define_constructor!(WidgetMessage:DesiredPosition => fn desired_position(Vector2<f32>), layout: false);
+    define_constructor!(WidgetMessage:Center => fn center(), layout: true);
+    define_constructor!(WidgetMessage:TopMost => fn topmost(), layout: false);
+    define_constructor!(WidgetMessage:Enabled => fn enabled(bool), layout: false);
+    define_constructor!(WidgetMessage:Name => fn name(String), layout: false);
+    define_constructor!(WidgetMessage:Row => fn row(usize), layout: false);
+    define_constructor!(WidgetMessage:Column => fn column(usize), layout: false);
+    define_constructor!(WidgetMessage:Cursor => fn cursor(Option<CursorIcon>), layout: false);
+    define_constructor!(WidgetMessage:ZIndex => fn z_index(usize), layout: false);
+    define_constructor!(WidgetMessage:HitTestVisibility => fn hit_test_visibility(bool), layout: false);
+    define_constructor!(WidgetMessage:Margin => fn margin(Thickness), layout: false);
+    define_constructor!(WidgetMessage:MinSize => fn min_size(Vector2<f32>), layout: false);
+    define_constructor!(WidgetMessage:MaxSize => fn max_size(Vector2<f32>), layout: false);
+    define_constructor!(WidgetMessage:HorizontalAlignment => fn horizontal_alignment(HorizontalAlignment), layout: false);
+    define_constructor!(WidgetMessage:VerticalAlignment => fn vertical_alignment(VerticalAlignment), layout: false);
+    define_constructor!(WidgetMessage:Opacity => fn opacity(Option<f32>), layout: false);
+
+    // Internal messages. Do not use.
+    define_constructor!(WidgetMessage:GotFocus => fn got_focus(), layout: false);
+    define_constructor!(WidgetMessage:LostFocus => fn lost_focus(), layout: false);
+    define_constructor!(WidgetMessage:MouseDown => fn mouse_down(pos: Vector2<f32>, button: MouseButton), layout: false);
+    define_constructor!(WidgetMessage:MouseUp => fn mouse_up(pos: Vector2<f32>, button: MouseButton), layout: false);
+    define_constructor!(WidgetMessage:MouseMove => fn mouse_move(pos: Vector2<f32>, state: MouseState), layout: false);
+    define_constructor!(WidgetMessage:MouseWheel => fn mouse_wheel(pos: Vector2<f32>, amount: f32), layout: false);
+    define_constructor!(WidgetMessage:MouseLeave => fn mouse_leave(), layout: false);
+    define_constructor!(WidgetMessage:MouseEnter => fn mouse_enter(), layout: false);
+    define_constructor!(WidgetMessage:Text => fn text(char), layout: false);
+    define_constructor!(WidgetMessage:KeyDown => fn key_down(KeyCode), layout: false);
+    define_constructor!(WidgetMessage:KeyUp => fn key_up(KeyCode), layout: false);
+    define_constructor!(WidgetMessage:DragStarted => fn drag_started(Handle<UiNode>), layout: false);
+    define_constructor!(WidgetMessage:DragOver => fn drag_over(Handle<UiNode>), layout: false);
+    define_constructor!(WidgetMessage:Drop => fn drop(Handle<UiNode>), layout: false);
+}
+
 #[derive(Debug, Clone)]
-pub struct Widget<M: MessageData, C: Control<M, C>> {
-    pub(in crate) handle: Handle<UINode<M, C>>,
+pub struct Widget {
+    pub(in crate) handle: Handle<UiNode>,
     name: String,
     /// Desired position relative to parent node
     desired_local_position: Vector2<f32>,
@@ -43,8 +337,8 @@ pub struct Widget<M: MessageData, C: Control<M, C>> {
     /// Current visibility state
     visibility: bool,
     global_visibility: bool,
-    children: Vec<Handle<UINode<M, C>>>,
-    parent: Handle<UINode<M, C>>,
+    children: Vec<Handle<UiNode>>,
+    parent: Handle<UiNode>,
     /// Indices of commands in command buffer emitted by the node.
     pub(in crate) command_indices: RefCell<Vec<usize>>,
     pub(in crate) is_mouse_directly_over: bool,
@@ -54,15 +348,15 @@ pub struct Widget<M: MessageData, C: Control<M, C>> {
     allow_drop: bool,
     pub user_data: Option<Rc<dyn Any>>,
     draw_on_top: bool,
-    marker: PhantomData<M>,
     enabled: bool,
     cursor: Option<CursorIcon>,
-    opacity: f32,
-    tooltip: Handle<UINode<M, C>>,
+    opacity: Option<f32>,
+    tooltip: Handle<UiNode>,
     tooltip_time: f32,
-    context_menu: Handle<UINode<M, C>>,
+    context_menu: Handle<UiNode>,
     pub(in crate) preview_messages: bool,
     pub(in crate) handle_os_events: bool,
+    pub(in crate) layout_events_sender: Option<Sender<LayoutEvent>>,
 
     /// Layout. Interior mutability is a must here because layout performed in
     /// a series of recursive calls.
@@ -80,9 +374,9 @@ pub struct Widget<M: MessageData, C: Control<M, C>> {
     pub(in crate) clip_bounds: Cell<Rect<f32>>,
 }
 
-impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
+impl Widget {
     #[inline]
-    pub fn handle(&self) -> Handle<UINode<M, C>> {
+    pub fn handle(&self) -> Handle<UiNode> {
         self.handle
     }
 
@@ -108,6 +402,18 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
         self
     }
 
+    #[inline]
+    pub fn set_min_width(&mut self, value: f32) -> &mut Self {
+        self.min_size.x = value;
+        self
+    }
+
+    #[inline]
+    pub fn set_min_height(&mut self, value: f32) -> &mut Self {
+        self.min_size.y = value;
+        self
+    }
+
     #[inline]
     pub fn min_size(&self) -> Vector2<f32> {
         self.min_size
@@ -142,11 +448,19 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     #[inline]
     pub fn invalidate_measure(&self) {
         self.measure_valid.set(false);
+
+        if let Some(layout_events_sender) = self.layout_events_sender.as_ref() {
+            let _ = layout_events_sender.send(LayoutEvent::MeasurementInvalidated(self.handle));
+        }
     }
 
     #[inline]
     pub fn invalidate_arrange(&self) {
         self.arrange_valid.set(false);
+
+        if let Some(layout_events_sender) = self.layout_events_sender.as_ref() {
+            let _ = layout_events_sender.send(LayoutEvent::ArrangementInvalidated(self.handle));
+        }
     }
 
     #[inline]
@@ -246,7 +560,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub(in crate) fn add_child(&mut self, child: Handle<UINode<M, C>>, in_front: bool) {
+    pub(in crate) fn add_child(&mut self, child: Handle<UiNode>, in_front: bool) {
         self.invalidate_layout();
         if in_front && !self.children.is_empty() {
             self.children.insert(0, child)
@@ -256,7 +570,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline(always)]
-    pub fn children(&self) -> &[Handle<UINode<M, C>>] {
+    pub fn children(&self) -> &[Handle<UiNode>] {
         &self.children
     }
 
@@ -267,7 +581,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub(in crate) fn remove_child(&mut self, child: Handle<UINode<M, C>>) {
+    pub(in crate) fn remove_child(&mut self, child: Handle<UiNode>) {
         if let Some(i) = self.children.iter().position(|h| *h == child) {
             self.children.remove(i);
             self.invalidate_layout();
@@ -275,12 +589,12 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub fn parent(&self) -> Handle<UINode<M, C>> {
+    pub fn parent(&self) -> Handle<UiNode> {
         self.parent
     }
 
     #[inline]
-    pub fn set_parent(&mut self, parent: Handle<UINode<M, C>>) {
+    pub fn set_parent(&mut self, parent: Handle<UiNode>) {
         self.parent = parent;
     }
 
@@ -310,12 +624,6 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
         self.desired_local_position
     }
 
-    #[inline]
-    pub fn set_visibility(&mut self, visibility: bool) -> &mut Self {
-        self.visibility = visibility;
-        self
-    }
-
     #[inline]
     pub fn screen_bounds(&self) -> Rect<f32> {
         Rect::new(
@@ -326,11 +634,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
         )
     }
 
-    pub fn has_descendant(
-        &self,
-        node_handle: Handle<UINode<M, C>>,
-        ui: &UserInterface<M, C>,
-    ) -> bool {
+    pub fn has_descendant(&self, node_handle: Handle<UiNode>, ui: &UserInterface) -> bool {
         for child_handle in self.children.iter() {
             if *child_handle == node_handle {
                 return true;
@@ -349,11 +653,11 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
 
     /// Searches a node up on tree starting from given root that matches a criteria
     /// defined by a given func.
-    pub fn find_by_criteria_up<Func: Fn(&UINode<M, C>) -> bool>(
+    pub fn find_by_criteria_up<Func: Fn(&UiNode) -> bool>(
         &self,
-        ui: &UserInterface<M, C>,
+        ui: &UserInterface,
         func: Func,
-    ) -> Handle<UINode<M, C>> {
+    ) -> Handle<UiNode> {
         let mut parent_handle = self.parent;
         while parent_handle.is_some() {
             let parent_node = ui.nodes.borrow(parent_handle);
@@ -365,41 +669,9 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
         Handle::NONE
     }
 
-    pub fn is_measure_valid_with_descendants(&self, ui: &UserInterface<M, C>) -> bool {
-        let mut valid =
-            self.is_measure_valid() && self.prev_global_visibility == self.is_globally_visible();
-        if valid {
-            for &child in self.children.iter() {
-                valid &= ui.node(child).is_measure_valid_with_descendants(ui);
-                if !valid {
-                    break;
-                }
-            }
-        }
-        valid
-    }
-
-    pub fn is_arrange_valid_with_descendants(&self, ui: &UserInterface<M, C>) -> bool {
-        let mut valid =
-            self.is_arrange_valid() && self.prev_global_visibility == self.is_globally_visible();
-        if valid {
-            for &child in self.children.iter() {
-                valid &= ui.node(child).is_arrange_valid_with_descendants(ui);
-                if !valid {
-                    break;
-                }
-            }
-        }
-        valid
-    }
-
-    pub fn handle_routed_message(
-        &mut self,
-        _ui: &mut UserInterface<M, C>,
-        msg: &mut UiMessage<M, C>,
-    ) {
+    pub fn handle_routed_message(&mut self, _ui: &mut UserInterface, msg: &mut UiMessage) {
         if msg.destination() == self.handle() && msg.direction() == MessageDirection::ToWidget {
-            if let UiMessageData::Widget(msg) = &msg.data() {
+            if let Some(msg) = msg.data::<WidgetMessage>() {
                 match msg {
                     &WidgetMessage::Opacity(opacity) => self.opacity = opacity,
                     WidgetMessage::Background(background) => self.background = background.clone(),
@@ -463,10 +735,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
                         self.hit_test_visibility = *hit_test_visibility
                     }
                     &WidgetMessage::Visibility(visibility) => {
-                        if self.visibility != visibility {
-                            self.visibility = visibility;
-                            self.invalidate_layout();
-                        }
+                        self.set_visibility(visibility);
                     }
                     &WidgetMessage::DesiredPosition(pos) => {
                         if self.desired_local_position != pos {
@@ -531,7 +800,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     #[inline]
     pub fn measure_override(
         &self,
-        ui: &UserInterface<M, C>,
+        ui: &UserInterface,
         available_size: Vector2<f32>,
     ) -> Vector2<f32> {
         let mut size: Vector2<f32> = Vector2::default();
@@ -547,11 +816,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub fn arrange_override(
-        &self,
-        ui: &UserInterface<M, C>,
-        final_size: Vector2<f32>,
-    ) -> Vector2<f32> {
+    pub fn arrange_override(&self, ui: &UserInterface, final_size: Vector2<f32>) -> Vector2<f32> {
         let final_rect = Rect::new(0.0, 0.0, final_size.x, final_size.y);
 
         for &child in self.children.iter() {
@@ -569,7 +834,7 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub(in crate) fn set_children(&mut self, children: Vec<Handle<UINode<M, C>>>) {
+    pub(in crate) fn set_children(&mut self, children: Vec<Handle<UiNode>>) {
         self.invalidate_layout();
         self.children = children;
     }
@@ -606,11 +871,29 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
         self.global_visibility
     }
 
+    #[inline]
+    pub fn set_visibility(&mut self, visibility: bool) -> &mut Self {
+        if self.visibility != visibility {
+            self.visibility = visibility;
+            self.invalidate_layout();
+            if let Some(layout_events_sender) = self.layout_events_sender.as_ref() {
+                let _ = layout_events_sender.send(LayoutEvent::VisibilityChanged(self.handle));
+            }
+        }
+        self
+    }
+
     #[inline]
     pub fn visibility(&self) -> bool {
         self.visibility
     }
 
+    #[inline]
+    pub fn set_enabled(&mut self, enabled: bool) -> &mut Self {
+        self.enabled = enabled;
+        self
+    }
+
     #[inline]
     pub fn enabled(&self) -> bool {
         self.enabled
@@ -637,18 +920,25 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub fn opacity(&self) -> f32 {
+    pub fn set_opacity(&mut self, opacity: Option<f32>) -> &mut Self {
+        self.opacity = opacity;
+        self
+    }
+
+    #[inline]
+    pub fn opacity(&self) -> Option<f32> {
         self.opacity
     }
 
     #[inline]
-    pub fn tooltip(&self) -> Handle<UINode<M, C>> {
+    pub fn tooltip(&self) -> Handle<UiNode> {
         self.tooltip
     }
 
     #[inline]
-    pub fn set_tooltip(&mut self, tooltip: Handle<UINode<M, C>>) {
+    pub fn set_tooltip(&mut self, tooltip: Handle<UiNode>) -> &mut Self {
         self.tooltip = tooltip;
+        self
     }
 
     #[inline]
@@ -657,34 +947,36 @@ impl<M: MessageData, C: Control<M, C>> Widget<M, C> {
     }
 
     #[inline]
-    pub fn set_tooltip_time(&mut self, tooltip_time: f32) {
+    pub fn set_tooltip_time(&mut self, tooltip_time: f32) -> &mut Self {
         self.tooltip_time = tooltip_time;
+        self
     }
 
     #[inline]
-    pub fn context_menu(&self) -> Handle<UINode<M, C>> {
+    pub fn context_menu(&self) -> Handle<UiNode> {
         self.context_menu
     }
 
     #[inline]
     /// The context menu receives `PopupMessage`s for being displayed, and so should support those.
-    pub fn set_context_menu(&mut self, context_menu: Handle<UINode<M, C>>) {
+    pub fn set_context_menu(&mut self, context_menu: Handle<UiNode>) -> &mut Self {
         self.context_menu = context_menu;
+        self
     }
 }
 
 #[macro_export]
 macro_rules! define_widget_deref {
     ($ty: ty) => {
-        impl<M: MessageData, C: Control<M, C>> Deref for $ty {
-            type Target = Widget<M, C>;
+        impl Deref for $ty {
+            type Target = Widget;
 
             fn deref(&self) -> &Self::Target {
                 &self.widget
             }
         }
 
-        impl<M: MessageData, C: Control<M, C>> DerefMut for $ty {
+        impl DerefMut for $ty {
             fn deref_mut(&mut self) -> &mut Self::Target {
                 &mut self.widget
             }
@@ -692,7 +984,7 @@ macro_rules! define_widget_deref {
     };
 }
 
-pub struct WidgetBuilder<M: MessageData, C: Control<M, C>> {
+pub struct WidgetBuilder {
     pub name: String,
     pub width: f32,
     pub height: f32,
@@ -706,7 +998,7 @@ pub struct WidgetBuilder<M: MessageData, C: Control<M, C>> {
     pub row: usize,
     pub column: usize,
     pub margin: Thickness,
-    pub children: Vec<Handle<UINode<M, C>>>,
+    pub children: Vec<Handle<UiNode>>,
     pub is_hit_test_visible: bool,
     pub visibility: bool,
     pub z_index: usize,
@@ -716,21 +1008,21 @@ pub struct WidgetBuilder<M: MessageData, C: Control<M, C>> {
     pub draw_on_top: bool,
     pub enabled: bool,
     pub cursor: Option<CursorIcon>,
-    pub opacity: f32,
-    pub tooltip: Handle<UINode<M, C>>,
+    pub opacity: Option<f32>,
+    pub tooltip: Handle<UiNode>,
     pub tooltip_time: f32,
-    pub context_menu: Handle<UINode<M, C>>,
+    pub context_menu: Handle<UiNode>,
     pub preview_messages: bool,
     pub handle_os_events: bool,
 }
 
-impl<M: MessageData, C: Control<M, C>> Default for WidgetBuilder<M, C> {
+impl Default for WidgetBuilder {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
+impl WidgetBuilder {
     pub fn new() -> Self {
         Self {
             name: Default::default(),
@@ -756,9 +1048,9 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
             draw_on_top: false,
             enabled: true,
             cursor: None,
-            opacity: 1.0,
+            opacity: None,
             tooltip: Handle::default(),
-            tooltip_time: 1.0,
+            tooltip_time: 0.1,
             context_menu: Handle::default(),
             preview_messages: false,
             handle_os_events: false,
@@ -845,7 +1137,7 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
         self
     }
 
-    pub fn with_child(mut self, handle: Handle<UINode<M, C>>) -> Self {
+    pub fn with_child(mut self, handle: Handle<UiNode>) -> Self {
         if handle.is_some() {
             self.children.push(handle);
         }
@@ -857,11 +1149,8 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
         self
     }
 
-    pub fn with_children<'a, I: IntoIterator<Item = &'a Handle<UINode<M, C>>>>(
-        mut self,
-        children: I,
-    ) -> Self {
-        for &child in children.into_iter() {
+    pub fn with_children<I: IntoIterator<Item = Handle<UiNode>>>(mut self, children: I) -> Self {
+        for child in children.into_iter() {
             if child.is_some() {
                 self.children.push(child)
             }
@@ -904,12 +1193,18 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
         self
     }
 
-    pub fn with_opacity(mut self, opacity: f32) -> Self {
+    pub fn with_opacity(mut self, opacity: Option<f32>) -> Self {
         self.opacity = opacity;
         self
     }
 
-    pub fn with_tooltip(mut self, tooltip: Handle<UINode<M, C>>) -> Self {
+    /// Sets the desired tooltip for the node.
+    ///
+    /// ## Important
+    ///
+    /// The widget will **own** the tooltip, which means that when widget will be deleted, the
+    /// tooltip will be deleted too.
+    pub fn with_tooltip(mut self, tooltip: Handle<UiNode>) -> Self {
         if tooltip.is_some() {
             self.tooltip = tooltip;
         }
@@ -922,14 +1217,14 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
     }
 
     /// The context menu receives `PopupMessage`s for being displayed, and so should support those.
-    pub fn with_context_menu(mut self, context_menu: Handle<UINode<M, C>>) -> Self {
+    pub fn with_context_menu(mut self, context_menu: Handle<UiNode>) -> Self {
         if context_menu.is_some() {
             self.context_menu = context_menu;
         }
         self
     }
 
-    pub fn build(self) -> Widget<M, C> {
+    pub fn build(self) -> Widget {
         Widget {
             handle: Default::default(),
             name: self.name,
@@ -968,7 +1263,6 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
             allow_drop: self.allow_drop,
             user_data: self.user_data.clone(),
             draw_on_top: self.draw_on_top,
-            marker: PhantomData,
             enabled: self.enabled,
             cursor: self.cursor,
             clip_bounds: Cell::new(Default::default()),
@@ -978,6 +1272,7 @@ impl<M: MessageData, C: Control<M, C>> WidgetBuilder<M, C> {
             context_menu: self.context_menu,
             preview_messages: self.preview_messages,
             handle_os_events: self.handle_os_events,
+            layout_events_sender: None,
         }
     }
 }
